/* GNU dump extensions to tar.

   Copyright (C) 1988, 1992, 1993, 1994, 1996, 1997, 1999, 2000, 2001,
   2003, 2004, 2005, 2006 Free Software Foundation, Inc.

   This program is free software; you can redistribute it and/or modify it
   under the terms of the GNU General Public License as published by the
   Free Software Foundation; either version 2, or (at your option) any later
   version.

   This program is distributed in the hope that it will be useful, but
   WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General
   Public License for more details.

   You should have received a copy of the GNU General Public License along
   with this program; if not, write to the Free Software Foundation, Inc.,
   51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.  */

#include <system.h>
#include <getline.h>
#include <hash.h>
#include <quotearg.h>
#include "common.h"

/* Incremental dump specialities.  */

/* Which child files to save under a directory.  */
enum children {NO_CHILDREN, CHANGED_CHILDREN, ALL_CHILDREN};

/* Directory attributes.  */
struct directory
  {
    struct timespec mtime;      /* Modification time */
    dev_t device_number;	/* device number for directory */
    ino_t inode_number;		/* inode number for directory */
    char *contents;             /* Directory contents */
    enum children children;     /* what to save under this directory */
    bool nfs;                   /* is the directory mounted on nfs? */
    bool found;                 /* was the directory found on fs? */
    bool new;                   /* is it new? */
    char name[1];		/* file name of directory */
  };

static Hash_table *directory_table;

#if HAVE_ST_FSTYPE_STRING
  static char const nfs_string[] = "nfs";
# define NFS_FILE_STAT(st) (strcmp ((st).st_fstype, nfs_string) == 0)
#else
# define ST_DEV_MSB(st) (~ (dev_t) 0 << (sizeof (st).st_dev * CHAR_BIT - 1))
# define NFS_FILE_STAT(st) (((st).st_dev & ST_DEV_MSB (st)) != 0)
#endif

/* Calculate the hash of a directory.  */
static size_t
hash_directory (void const *entry, size_t n_buckets)
{
  struct directory const *directory = entry;
  return hash_string (directory->name, n_buckets);
}

/* Compare two directories for equality.  */
static bool
compare_directories (void const *entry1, void const *entry2)
{
  struct directory const *directory1 = entry1;
  struct directory const *directory2 = entry2;
  return strcmp (directory1->name, directory2->name) == 0;
}

static struct directory *
make_directory (const char *name)
{
  size_t namelen = strlen (name);
  size_t size = offsetof (struct directory, name) + namelen + 1;
  struct directory *directory = xmalloc (size);
  strcpy (directory->name, name);
  if (ISSLASH (directory->name[namelen-1]))
    directory->name[namelen-1] = 0;
  directory->new = false;
  return directory;
}
  
/* Create and link a new directory entry for directory NAME, having a
   device number DEV and an inode number INO, with NFS indicating
   whether it is an NFS device and FOUND indicating whether we have
   found that the directory exists.  */
static struct directory *
note_directory (char const *name, struct timespec mtime,
		dev_t dev, ino_t ino, bool nfs, bool found, char *contents)
{
  struct directory *directory = make_directory (name);

  directory->mtime = mtime;
  directory->device_number = dev;
  directory->inode_number = ino;
  directory->children = CHANGED_CHILDREN;
  directory->nfs = nfs;
  directory->found = found;
  directory->contents = contents;
  
  if (! ((directory_table
	  || (directory_table = hash_initialize (0, 0, hash_directory,
						 compare_directories, 0)))
	 && hash_insert (directory_table, directory)))
    xalloc_die ();

  return directory;
}

/* Return a directory entry for a given file NAME, or zero if none found.  */
static struct directory *
find_directory (char *name)
{
  if (! directory_table)
    return 0;
  else
    {
      struct directory *dir = make_directory (name);
      struct directory *ret = hash_lookup (directory_table, dir);
      free (dir);
      return ret;
    }
}

void
update_parent_directory (const char *name)
{
  struct directory *directory;
  char *p, *name_buffer;

  p = dir_name (name);
  directory = find_directory (p);
  if (directory)
    {
      struct stat st;
      if (deref_stat (dereference_option, p, &st) != 0)
	stat_diag (name);
      else
	directory->mtime = get_stat_mtime (&st);
    }
  free (p);
}

static struct directory *
procdir (char *name_buffer, struct stat *stat_data,
	 dev_t device,
	 enum children children,
	 bool verbose)
{
  struct directory *directory;
  bool nfs = NFS_FILE_STAT (*stat_data);
  struct name *np;

  if ((directory = find_directory (name_buffer)) != NULL)
    {
      /* With NFS, the same file can have two different devices
	 if an NFS directory is mounted in multiple locations,
	 which is relatively common when automounting.
	 To avoid spurious incremental redumping of
	 directories, consider all NFS devices as equal,
	 relying on the i-node to establish differences.  */

      if (! (((directory->nfs & nfs)
	      || directory->device_number == stat_data->st_dev)
	     && directory->inode_number == stat_data->st_ino))
	{
	  if (verbose)
	    WARN ((0, 0, _("%s: Directory has been renamed"),
		   quotearg_colon (name_buffer)));
	  directory->children = ALL_CHILDREN;
	  directory->contents = NULL; /* Make sure makedumpdir marks
					 all entries as new */
	  directory->nfs = nfs;
	  directory->device_number = stat_data->st_dev;
	  directory->inode_number = stat_data->st_ino;
	}
      else if (listed_incremental_option && !directory->new)
	/* Newer modification time can mean that new files were
	   created in the directory or some of the existing files
	   were renamed. */
	directory->children =
	  timespec_cmp (get_stat_mtime (stat_data), directory->mtime) > 0
	  ? ALL_CHILDREN : CHANGED_CHILDREN;

      directory->found = true;
    }
  else
    {
      if (verbose)
	WARN ((0, 0, _("%s: Directory is new"),
	       quotearg_colon (name_buffer)));
      directory = note_directory (name_buffer,
				  get_stat_mtime(stat_data),
				  stat_data->st_dev,
				  stat_data->st_ino,
				  nfs,
				  true,
				  NULL);

      directory->children =
	(listed_incremental_option
	 || (OLDER_STAT_TIME (*stat_data, m)
	     || (after_date_option
		 && OLDER_STAT_TIME (*stat_data, c))))
	? ALL_CHILDREN
	: CHANGED_CHILDREN;
      directory->new = true;
    }

  /* If the directory is on another device and --one-file-system was given,
     omit it... */
  if (one_file_system_option && device != stat_data->st_dev
      /* ... except if it was explicitely given in the command line */
      && !((np = name_scan (name_buffer, true)) && np->explicit))
    directory->children = NO_CHILDREN;
  else if (children == ALL_CHILDREN)
    directory->children = ALL_CHILDREN;

  return directory;
}

/* Locate NAME in the dumpdir array DUMP.
   Return pointer to the slot in the array, or NULL if not found */
const char *
dumpdir_locate (const char *dump, const char *name)
{
  if (dump)
    while (*dump)
      {
	int rc = strcmp (dump + 1, name);
	if (rc == 0)
	  return dump;
	if (rc > 1)
	  break;
	dump += strlen (dump) + 1;
      }
  return NULL;
}

/* Return size in bytes of the dumpdir array P */
size_t
dumpdir_size (const char *p)
{
  size_t totsize = 0;

  while (*p)
    {
      size_t size = strlen (p) + 1;
      totsize += size;
      p += size;
    }
  return totsize + 1;
}

static int
compare_dirnames (const void *first, const void *second)
{
  return strcmp (*(const char**)first, *(const char**)second);
}

/* Compare dumpdir array DUMP with directory listing DIR and build a
   new dumpdir template.

   DUMP must be a valid dumpdir array.
   DIR must be returned by a previous call to savedir().

   File names in both DUMP and DIR must be sorted alphabetically.

   Created dumpdir template is returned in NEW_DUMP_PTR. Each entry is
   prefixed with ' ' if it was present in DUMP and with 'a' otherwise.

   Return true if DUMP and DIR differ. */
static bool
makedumpdir (const char *dump, const char *dir, char **new_dump_ptr)
{
  size_t i, dirsize, len;
  const char *p;
  char const **array;
  char *new_dump;
  bool diff = false;
  
  dirsize = 0;
  len = 0;
  for (p = dir; *p; p += strlen (p) + 1, dirsize++)
    len += strlen (p) + 2;
  len++;

  array = xcalloc (dirsize, sizeof array[0]);
  for (i = 0, p = dir; *p; p += strlen (p) + 1, i++)
    array[i] = p;

  qsort (array, dirsize, sizeof (array[0]), compare_dirnames);

  new_dump = xmalloc (len);
  *new_dump_ptr = new_dump;
  
  for (i = 0; i < dirsize; i++)
    {
      const char *loc = dumpdir_locate (dump, array[i]);
      if (loc)
	{
	  *new_dump++ = 'Y';
	  dump = loc + strlen (loc) + 1;
	}
      else
	{
	  diff = true;
	  *new_dump++ = 'a'; /* New entry */
	}
      for (p = array[i]; (*new_dump++ = *p++); )
	;
    }
  if (!diff)
    diff = !dump || *dump != 0;
  *new_dump = 0;
  free (array);
  return diff;
}

/* Recursively scan the given directory. */
static char *
scan_directory (char *dir_name, dev_t device)
{
  char *dirp = savedir (dir_name);	/* for scanning directory */
  char *name_buffer;		/* directory, `/', and directory member */
  size_t name_buffer_size;	/* allocated size of name_buffer, minus 2 */
  size_t name_length;		/* used length in name_buffer */
  enum children children;
  struct stat stat_data;
  struct directory *directory;
  char *new_contents;
  
  if (! dirp)
    savedir_error (dir_name);

  name_buffer_size = strlen (dir_name) + NAME_FIELD_SIZE;
  name_buffer = xmalloc (name_buffer_size + 2);
  strcpy (name_buffer, dir_name);
  if (! ISSLASH (dir_name[strlen (dir_name) - 1]))
    strcat (name_buffer, "/");
  name_length = strlen (name_buffer);

  if (deref_stat (dereference_option, name_buffer, &stat_data))
    {
      stat_diag (name_buffer);
      /* FIXME: used to be 
           children = CHANGED_CHILDREN;
	 but changed to: */
      free (name_buffer);
      free (dirp);
      return NULL;
    }
  else
    directory = procdir (name_buffer, &stat_data, device, NO_CHILDREN,
			 false);

  if (dirp && directory->children != NO_CHILDREN)
    {
      char  *entry;	/* directory entry being scanned */
      size_t entrylen;	/* length of directory entry */
      char *contents;
      bool changed = makedumpdir (directory->contents, dirp, &contents);

      if (changed)
	directory->children = CHANGED_CHILDREN;

      children = directory->children;

      for (entry = contents;
	   (entrylen = strlen (entry)) != 0;
	   entry += entrylen + 1)
	{
	  if (name_buffer_size <= entrylen - 1 + name_length)
	    {
	      do
		name_buffer_size += NAME_FIELD_SIZE;
	      while (name_buffer_size <= entrylen - 1 + name_length);
	      name_buffer = xrealloc (name_buffer, name_buffer_size + 2);
	    }
	  strcpy (name_buffer + name_length, entry + 1);
	  
	  if (excluded_name (name_buffer))
	    *entry = 'N';
	  else
	    {
	      if (deref_stat (dereference_option, name_buffer, &stat_data))
		{
		  stat_diag (name_buffer);
		  continue;
		}
	      
	      if (S_ISDIR (stat_data.st_mode))
		{
		  procdir (name_buffer, &stat_data, device, children,
			   verbose_option);
		  *entry = 'D';
		}
	      
	      else if (one_file_system_option && device != stat_data.st_dev)
		*entry = 'N';

	      else if (*entry == 'a') /* New entry */
		*entry = 'Y';
	      
	      /* FIXME: if (S_ISHIDDEN (stat_data.st_mode))?? */
	      
	      else if (children == CHANGED_CHILDREN
		       && OLDER_STAT_TIME (stat_data, m)
		       && (!after_date_option
			   || OLDER_STAT_TIME (stat_data, c)))
		*entry = 'N';
	      else
		*entry = 'Y';
	    }
	}
      /* FIXME: Initial directory->contents was allocated in an obstack and
         therefore not freed */
      directory->contents = contents;
    }
  
  free (name_buffer);
  if (dirp)
    free (dirp);

  return directory->contents;
}

char *
get_directory_contents (char *dir_name, dev_t device)
{
  return scan_directory (dir_name, device);
}



static FILE *listed_incremental_stream;

/* Version of incremental format snapshots (directory files) used by this
   tar. Currently it is supposed to be a single decimal number. 0 means
   incremental snapshots as per tar version before 1.15.2.

   The current tar version supports incremental versions from
   0 up to TAR_INCREMENTAL_VERSION, inclusive.
   It is able to create only snapshots of TAR_INCREMENTAL_VERSION */

#define TAR_INCREMENTAL_VERSION 2

/* Read incremental snapshot formats 0 and 1 */
static void
read_incr_db_01 (int version, const char *initbuf)
{
  int n;
  uintmax_t u;
  time_t t = u;
  char *buf = 0;
  size_t bufsize;
  char *ebuf;
  long lineno = 1;

  if (version == 1)
    {
      if (getline (&buf, &bufsize, listed_incremental_stream) <= 0)
	{
	  read_error (listed_incremental_option);
	  free (buf);
	  return;
	}
      ++lineno;
    }
  else
    {
      buf = strdup (initbuf);
      bufsize = strlen (buf) + 1;
    }
  
  t = u = (errno = 0, strtoumax (buf, &ebuf, 10));
  if (buf == ebuf || (u == 0 && errno == EINVAL))
    ERROR ((0, 0, "%s:%ld: %s",
	    quotearg_colon (listed_incremental_option),
	    lineno,
	    _("Invalid time stamp")));
  else if (t != u)
    ERROR ((0, 0, "%s:%ld: %s",
	    quotearg_colon (listed_incremental_option),
	    lineno,
	    _("Time stamp out of range")));
  else if (version == 1)
    {
      newer_mtime_option.tv_sec = t;

      t = u = (errno = 0, strtoumax (buf, &ebuf, 10));
      if (buf == ebuf || (u == 0 && errno == EINVAL))
	ERROR ((0, 0, "%s:%ld: %s",
		quotearg_colon (listed_incremental_option),
		lineno,
		_("Invalid time stamp")));
      else if (t != u)
	ERROR ((0, 0, "%s:%ld: %s",
		quotearg_colon (listed_incremental_option),
		lineno,
		_("Time stamp out of range")));
      newer_mtime_option.tv_nsec = t;
    }
  else
    {
      /* pre-1 incremental format does not contain nanoseconds */
      newer_mtime_option.tv_sec = t;
      newer_mtime_option.tv_nsec = 0;
    }

  while (0 < (n = getline (&buf, &bufsize, listed_incremental_stream)))
    {
      dev_t dev;
      ino_t ino;
      bool nfs = buf[0] == '+';
      char *strp = buf + nfs;
      struct timespec mtime;
      
      lineno++;
      
      if (buf[n - 1] == '\n')
	buf[n - 1] = '\0';
      
      if (version == 1)
	{
	  errno = 0;
	  mtime.tv_sec = u = strtoumax (strp, &ebuf, 10);
	  if (!isspace (*ebuf))
	    ERROR ((0, 0, "%s:%ld: %s",
		    quotearg_colon (listed_incremental_option), lineno,
		    _("Invalid modification time (seconds)")));
	  else if (mtime.tv_sec != u)
	    ERROR ((0, 0, "%s:%ld: %s",
		    quotearg_colon (listed_incremental_option), lineno,
		    _("Modification time (seconds) out of range")));
	  strp = ebuf;
	  
	  errno = 0;
	  mtime.tv_nsec = u = strtoumax (strp, &ebuf, 10);
	  if (!isspace (*ebuf))
	    ERROR ((0, 0, "%s:%ld: %s",
		    quotearg_colon (listed_incremental_option), lineno,
		    _("Invalid modification time (nanoseconds)")));
	  else if (mtime.tv_nsec != u)
	    ERROR ((0, 0, "%s:%ld: %s",
		    quotearg_colon (listed_incremental_option), lineno,
		    _("Modification time (nanoseconds) out of range")));
	  strp = ebuf;
	}
      else
	memset (&mtime, 0, sizeof mtime);
      
      errno = 0;
      dev = u = strtoumax (strp, &ebuf, 10);
      if (!isspace (*ebuf))
	ERROR ((0, 0, "%s:%ld: %s",
		quotearg_colon (listed_incremental_option), lineno,
		_("Invalid device number")));
      else if (dev != u)
	ERROR ((0, 0, "%s:%ld: %s",
		quotearg_colon (listed_incremental_option), lineno,
		_("Device number out of range")));
      strp = ebuf;

      errno = 0;
      ino = u = strtoumax (strp, &ebuf, 10);
      if (!isspace (*ebuf))
	ERROR ((0, 0, "%s:%ld: %s",
		quotearg_colon (listed_incremental_option), lineno,
		_("Invalid inode number")));
      else if (ino != u)
	ERROR ((0, 0, "%s:%ld: %s",
		quotearg_colon (listed_incremental_option), lineno,
		_("Inode number out of range")));
      strp = ebuf;
      
      strp++;
      unquote_string (strp);
      note_directory (strp, mtime, dev, ino, nfs, false, NULL);
    }
  free (buf);
}

/* Read a nul-terminated string from FP and store it in STK.
   Store the number of bytes read (including nul terminator) in PCOUNT.

   Return the last character read or EOF on end of file. */
static int
read_obstack (FILE *fp, struct obstack *stk, size_t *pcount)
{
  int c;
  size_t i;

  for (i = 0, c = getc (fp); c != EOF && c != 0; c = getc (fp), i++)
    obstack_1grow (stk, c);
  obstack_1grow (stk, 0);

  *pcount = i;
  return c;
}

/* Read from file FP a nul-terminated string and convert it to
   uintmax_t. Return the resulting value in PVAL.

   Throw fatal error if the string cannot be converted.
   
   Return the last character read or EOF on end of file. */

static int
read_num (FILE *fp, uintmax_t *pval)
{
  int c;
  size_t i;
  char buf[UINTMAX_STRSIZE_BOUND], *ep;

  for (i = 0, c = getc (fp); c != EOF && c != 0; c = getc (fp), i++)
    {
      if (i == sizeof buf)
	FATAL_ERROR ((0, 0, _("Field too long while reading snapshot file")));
      buf[i] = c;
    }
  buf[i] = 0;
  *pval = strtoumax (buf, &ep, 10);
  if (*ep)
    FATAL_ERROR ((0, 0, _("Unexpected field value in snapshot file")));
  return c;
}  

/* Read incremental snapshot format 2 */
static void
read_incr_db_2 ()
{
  uintmax_t u;
  struct obstack stk;
  
  obstack_init (&stk);

  if (read_num (listed_incremental_stream, &u))
    FATAL_ERROR ((0, 0, "%s: %s",
		  quotearg_colon (listed_incremental_option), 
		  _("Error reading time stamp")));
  newer_mtime_option.tv_sec = u;
  if (newer_mtime_option.tv_sec != u)
    FATAL_ERROR ((0, 0, "%s: %s",
		  quotearg_colon (listed_incremental_option),
		  _("Time stamp out of range")));

  if (read_num (listed_incremental_stream, &u))
    FATAL_ERROR ((0, 0, "%s: %s",
		  quotearg_colon (listed_incremental_option), 
		  _("Error reading time stamp")));
  newer_mtime_option.tv_nsec = u;
  if (newer_mtime_option.tv_nsec != u)
    FATAL_ERROR ((0, 0, "%s: %s",
		  quotearg_colon (listed_incremental_option),
		  _("Time stamp out of range")));

  for (;;)
    {
      struct timespec mtime;
      dev_t dev;
      ino_t ino;
      bool nfs;
      char *name;
      char *content;
      size_t s;
      
      if (read_num (listed_incremental_stream, &u))
	return; /* Normal return */

      nfs = u;
      
      if (read_num (listed_incremental_stream, &u))
	break;
      mtime.tv_sec = u;
      if (mtime.tv_sec != u)
	FATAL_ERROR ((0, 0, "%s: %s",
		      quotearg_colon (listed_incremental_option), 
		      _("Modification time (seconds) out of range")));
      
      if (read_num (listed_incremental_stream, &u))
	break;
      mtime.tv_nsec = u;
      if (mtime.tv_nsec != u)
	FATAL_ERROR ((0, 0, "%s: %s",
		      quotearg_colon (listed_incremental_option), 
		      _("Modification time (nanoseconds) out of range")));

      if (read_num (listed_incremental_stream, &u))
	break;
      dev = u;
      if (dev != u)
	FATAL_ERROR ((0, 0, "%s: %s",
		      quotearg_colon (listed_incremental_option), 
		      _("Device number out of range")));

      if (read_num (listed_incremental_stream, &u))
	break;
      ino = u;
      if (ino != u)
	FATAL_ERROR ((0, 0, "%s: %s",
		      quotearg_colon (listed_incremental_option), 
		      _("Inode number out of range")));

      if (read_obstack (listed_incremental_stream, &stk, &s))
	break;

      name = obstack_finish (&stk);

      while (read_obstack (listed_incremental_stream, &stk, &s) == 0 && s > 1)
	;
      if (getc (listed_incremental_stream) != 0)
	FATAL_ERROR ((0, 0, "%s: %s",
		      quotearg_colon (listed_incremental_option), 
		      _("Missing record terminator")));
      
      content = obstack_finish (&stk);
      note_directory (name, mtime, dev, ino, nfs, false, content);
    }
  FATAL_ERROR ((0, 0, "%s: %s",
		quotearg_colon (listed_incremental_option), 
		_("Unexpected EOF")));
}

/* Read incremental snapshot file (directory file).
   If the file has older incremental version, make sure that it is processed
   correctly and that tar will use the most conservative backup method among
   possible alternatives (i.e. prefer ALL_CHILDREN over CHANGED_CHILDREN,
   etc.) This ensures that the snapshots are updated to the recent version
   without any loss of data. */
void
read_directory_file (void)
{
  int fd;
  char *buf = 0;
  size_t bufsize;
  long lineno = 1;

  /* Open the file for both read and write.  That way, we can write
     it later without having to reopen it, and don't have to worry if
     we chdir in the meantime.  */
  fd = open (listed_incremental_option, O_RDWR | O_CREAT, MODE_RW);
  if (fd < 0)
    {
      open_error (listed_incremental_option);
      return;
    }

  listed_incremental_stream = fdopen (fd, "r+");
  if (! listed_incremental_stream)
    {
      open_error (listed_incremental_option);
      close (fd);
      return;
    }

  if (0 < getline (&buf, &bufsize, listed_incremental_stream))
    {
      char *ebuf;
      int incremental_version;

      if (strncmp (buf, PACKAGE_NAME, sizeof PACKAGE_NAME - 1) == 0)
	{
	  ebuf = buf + sizeof PACKAGE_NAME - 1;
	  if (*ebuf++ != '-')
	    ERROR((1, 0, _("Bad incremental file format")));
	  for (; *ebuf != '-'; ebuf++)
	    if (!*ebuf)
	      ERROR((1, 0, _("Bad incremental file format")));

	  incremental_version = (errno = 0, strtoumax (ebuf+1, &ebuf, 10));
	}
      else
	incremental_version = 0;

      switch (incremental_version)
	{
	case 0:
	case 1:
	  read_incr_db_01 (incremental_version, buf);
	  break;

	case TAR_INCREMENTAL_VERSION:
	  read_incr_db_2 ();
	  break;
	  
	default:
	  ERROR ((1, 0, _("Unsupported incremental format version: %d"),
		  incremental_version));
	}
      
    }

  if (ferror (listed_incremental_stream))
    read_error (listed_incremental_option);
  if (buf)
    free (buf);
}

/* Output incremental data for the directory ENTRY to the file DATA.
   Return nonzero if successful, preserving errno on write failure.  */
static bool
write_directory_file_entry (void *entry, void *data)
{
  struct directory const *directory = entry;
  FILE *fp = data;

  if (directory->found)
    {
      char buf[UINTMAX_STRSIZE_BOUND];
      char *s;

      s = directory->nfs ? "1" : "0";
      fwrite (s, 2, 1, fp);
      s = umaxtostr (directory->mtime.tv_sec, buf);
      fwrite (s, strlen (s) + 1, 1, fp);
      s = umaxtostr (directory->mtime.tv_nsec, buf);
      fwrite (s, strlen (s) + 1, 1, fp);
      s = umaxtostr (directory->device_number, buf);
      fwrite (s, strlen (s) + 1, 1, fp);
      s = umaxtostr (directory->inode_number, buf);
      fwrite (s, strlen (s) + 1, 1, fp);

      fwrite (directory->name, strlen (directory->name) + 1, 1, fp);
      if (directory->contents)
	fwrite (directory->contents, dumpdir_size (directory->contents),
		1, fp);
      else
	fwrite ("\0", 1, 1, fp);
      fwrite ("\0", 1, 1, fp);
    }

  return ! ferror (fp);
}

void
write_directory_file (void)
{
  FILE *fp = listed_incremental_stream;
  char buf[UINTMAX_STRSIZE_BOUND];
  char *s;

  if (! fp)
    return;

  if (fseek (fp, 0L, SEEK_SET) != 0)
    seek_error (listed_incremental_option);
  if (sys_truncate (fileno (fp)) != 0)
    truncate_error (listed_incremental_option);

  fprintf (fp, "%s-%s-%d\n", PACKAGE_NAME, PACKAGE_VERSION,
	   TAR_INCREMENTAL_VERSION);

  s = umaxtostr (start_time.tv_sec, buf);
  fwrite (s, strlen (s) + 1, 1, fp);
  s = umaxtostr (start_time.tv_nsec, buf);
  fwrite (s, strlen (s) + 1, 1, fp);

  if (! ferror (fp) && directory_table)
    hash_do_for_each (directory_table, write_directory_file_entry, fp);

  if (ferror (fp))
    write_error (listed_incremental_option);
  if (fclose (fp) != 0)
    close_error (listed_incremental_option);
}


/* Restoration of incremental dumps.  */

static void
get_gnu_dumpdir (struct tar_stat_info *stat_info)
{
  size_t size;
  size_t copied;
  union block *data_block;
  char *to;
  char *archive_dir;

  size = stat_info->stat.st_size;

  archive_dir = xmalloc (size);
  to = archive_dir;

  set_next_block_after (current_header);
  mv_begin (stat_info);

  for (; size > 0; size -= copied)
    {
      mv_size_left (size);
      data_block = find_next_block ();
      if (!data_block)
	ERROR ((1, 0, _("Unexpected EOF in archive")));
      copied = available_space_after (data_block);
      if (copied > size)
	copied = size;
      memcpy (to, data_block->buffer, copied);
      to += copied;
      set_next_block_after ((union block *)
			    (data_block->buffer + copied - 1));
    }

  mv_end ();

  stat_info->dumpdir = archive_dir;
  stat_info->skipped = true; /* For skip_member() and friends
				to work correctly */
}

/* Return T if STAT_INFO represents a dumpdir archive member.
   Note: can invalidate current_header. It happens if flush_archive()
   gets called within get_gnu_dumpdir() */
bool
is_dumpdir (struct tar_stat_info *stat_info)
{
  if (stat_info->is_dumpdir && !stat_info->dumpdir)
    get_gnu_dumpdir (stat_info);
  return stat_info->is_dumpdir;
}

/* Examine the directories under directory_name and delete any
   files that were not there at the time of the back-up. */
void
purge_directory (char const *directory_name)
{
  char *current_dir;
  char *cur, *arc;

  if (!is_dumpdir (&current_stat_info))
    {
      skip_member ();
      return;
    }

  current_dir = savedir (directory_name);

  if (!current_dir)
    {
      /* The directory doesn't exist now.  It'll be created.  In any
	 case, we don't have to delete any files out of it.  */

      skip_member ();
      return;
    }

  for (cur = current_dir; *cur; cur += strlen (cur) + 1)
    {
      for (arc = current_stat_info.dumpdir; *arc; arc += strlen (arc) + 1)
	{
	  arc++;
	  if (!strcmp (arc, cur))
	    break;
	}
      if (*arc == '\0')
	{
	  struct stat st;
	  char *p = new_name (directory_name, cur);

	  if (deref_stat (false, p, &st))
	    {
	      stat_diag (p);
	      WARN((0, 0, _("%s: Not purging directory: unable to stat"),
		    quotearg_colon (p)));
	      continue;
	    }
	  else if (one_file_system_option && st.st_dev != root_device)
	    {
	      WARN((0, 0,
		    _("%s: directory is on a different device: not purging"),
		    quotearg_colon (p)));
	      continue;
	    }

	  if (! interactive_option || confirm ("delete", p))
	    {
	      if (verbose_option)
		fprintf (stdlis, _("%s: Deleting %s\n"),
			 program_name, quote (p));
	      if (! remove_any_file (p, RECURSIVE_REMOVE_OPTION))
		{
		  int e = errno;
		  ERROR ((0, e, _("%s: Cannot remove"), quotearg_colon (p)));
		}
	    }
	  free (p);
	}

    }
  free (current_dir);
}

void
list_dumpdir (char *buffer, size_t size)
{
  while (size)
    {
      switch (*buffer)
	{
	case 'Y':
	case 'N':
	case 'D':
	  fprintf (stdlis, "%c ", *buffer);
	  buffer++;
	  size--;
	  break;

	case 0:
	  fputc ('\n', stdlis);
	  buffer++;
	  size--;
	  break;

	default:
	  fputc (*buffer, stdlis);
	  buffer++;
	  size--;
	}
    }
}
